/***********************************************************************
 * File: fair_copy-test.c
 * 	(Extracted from
 *	   "$Id: fair_copy-test.c,v 1.2 2007/10/29 06:40:27 Administrator Exp Administrator $")
 *
 * Abstracts:
 *  This C source implements the test program for `fair_copy()' API.
 *
 * Copyright (C) 2004-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "fair_copy.h"

static void print_in_text_rectangle_wrap_around(char *sp
						, size_t col_width
						)
{
	size_t	i, j;
	char	*p = sp;

	for (i = 1; *sp != '\0'; ++i) {
		printf("\n%3lu: ", (unsigned long)i)
		; (void)fflush(stdout);
		for (j = 0
		; (*sp != '\0') && j < col_width
		; ++sp, ++j) {
			if (*sp == '\n') {
				++sp;
				break;
			}
			printf("%c", *sp); (void)fflush(stdout);
		}
	}
	printf("\n==============================\n"
		"%s"
		"\n==============================\n", p
		); (void)fflush(stdout);

	return;
}


static void print_in_text_rectangle_line_breaking(char *sp
						, size_t col_width
						)
{
	size_t	i, j;
	char	*p = sp;

	for (i = 1; *sp != '\0'; ++i) {
		printf("\n%3lu: ", (unsigned long)i)
		; (void)fflush(stdout);
		for (j = 0
		; ((*sp != '\0') && *sp != '\n') && j < col_width
		; ++sp, ++j) {
			printf("%c", *sp); (void)fflush(stdout);
		}
		if (*sp == '\n') {
			++sp;
		}
	}
	printf("\n==============================\n"
		"%s"
		"\n==============================\n", p
		); (void)fflush(stdout);

	return;
}


int main(int ac, char **av)
{
	int	result;
	char	src[4096], dst[4096], *sp = src;
	size_t	rest = sizeof(src) / 2, processed_rows
		, c, cols[] = { 1, 2, 5, 10, 20, 50, 100 }, rows;
	FILE	*fp = stdin;

	if (ac == 2) {
		if ((fp = fopen(av[1], "r")) == NULL) {
			fprintf(stderr
				, "***FAILED to open \"%s\" to read.\n"
				,			av[1]
				); (void)fflush(stderr);
			exit(2);
		}
	}

	while ((rest > 0) && fgets(sp, (int)rest, fp) != NULL) {
		size_t	read_sz = strlen(sp);
		sp	+= read_sz;
		rest	-= read_sz;
	}

	if (ac == 2) {
		(void)fclose(fp);
	}

	for (c = 0; c < sizeof(cols) / sizeof(cols[0]); ++c) {
		Fair_copy_line_control	linectl;

		for (linectl = Fair_copy_line_breaking
		; linectl <= Fair_copy_wrap_around
		; linectl = (Fair_copy_line_control)(linectl + 1)) {
			size_t	dsz;
			
			if ((c == 0)
			&& linectl == Fair_copy_line_breaking) {
				printf("linectl = %d."
					"\nsrc is...\n[%s]"
					, (int)linectl
					, src
					); (void)fflush(stdout);
			}

			rows	= 0;
			processed_rows	= 0L;
			result	= fair_copy(src
					, linectl
					, rows
					, cols[c]
					, (size_t)0
					, NULL
					, &processed_rows
					);
			if (result != FAIR_COPY_NORMAL_END) {
				fprintf(stderr
					, "**FAILED: fair_copy(%p, %d,"
					" %lu, %lu, %ld, %p, %p) "
					"returns %d.\n"
					, src, linectl
					, (unsigned long)rows
					, (unsigned long)cols[c]
					, 0L, dst
					, (void *)&processed_rows
					, result
					); (void)fflush(stderr);
				continue;
			}

			printf("\n<<< Before text {%s} (linectl = %d)\n"
				,		av[1],		linectl
				); (void)fflush(stdout);
			printf("%3lu: ----+----|----+----|----+----|"
				     "----+----|----+----|----+----|"
				     "----+----|----+----|----+----|"
				     "----+----|"
				, (unsigned long)(cols[c])
				); (void)fflush(stdout);
			(*((linectl == Fair_copy_line_breaking)?
				  print_in_text_rectangle_line_breaking
				: print_in_text_rectangle_wrap_around
					))(src, cols[c]);
			printf("[processed_rows = %ld, "
				"strlen(src) = %u]\n\n"
				, (long)processed_rows
				, (unsigned)strlen(src)
				); (void)fflush(stdout);

			rows	= 100 / cols[c];
			processed_rows	= 0L;
			dsz = rows * cols[c]
				+ ((linectl == Fair_copy_line_breaking)?
							rows: 1);
			result	= fair_copy(src
					, linectl
					, rows
					, cols[c]
					, dsz
					, dst
					, &processed_rows
					);
			if (result != FAIR_COPY_NORMAL_END) {
				fprintf(stderr
					, "**FAILED: fair_copy(%p, %d,"
					" %lu, %lu, %ld, %p, %p) "
					"returns %d.\n"
					, src, linectl
					, (unsigned long)rows
					, (unsigned long)cols[c]
					, (long)dsz
					, dst, (void *)&processed_rows
					, result
					); (void)fflush(stderr);
			}

			printf("\n>>> Processed text {%s} "
				"(linectl = %d)\n"
				,			av[1]
				,	linectl
				); (void)fflush(stdout);
			printf("%3lu: ----+----|----+----|----+----|"
				     "----+----|----+----|----+----|"
				     "----+----|----+----|----+----|"
				     "----+----|"
				, (unsigned long)cols[c]
				); (void)fflush(stdout);
			(*((linectl == Fair_copy_line_breaking)?
				  print_in_text_rectangle_line_breaking
				: print_in_text_rectangle_wrap_around
					))(dst, cols[c]);
			printf("[processed_rows = %ld, "
				"strlen(dst) = %u)]\n\n"
				, (long)processed_rows
				, (unsigned)strlen(dst)
				); (void)fflush(stdout);
		}
	}

	if ((fp = fopen("test.out.txt", "w")) != NULL) {
		fprintf(fp, "%s", dst); (void)fflush(fp)
		; (void)fclose(fp);
	}

	exit(0);
}
